@@ -97,19 +97,19 @@
 
 #define HH_CONFIG_THEME_MONOCHROMATIC   "monochromatic"
 #define HH_CONFIG_THEME_HICOLOR         "hicolor"
-#define HH_CONFIG_CASE       "casesensitive"
-#define HH_CONFIG_REGEXP     "regexp"
-#define HH_CONFIG_SUBSTRING  "substring"
-#define HH_CONFIG_KEYWORDS   "keywords"
-#define HH_CONFIG_SORTING    "rawhistory"
-#define HH_CONFIG_FAVORITES  "favorites"
-#define HH_CONFIG_NOCONFIRM  "noconfirm"
-// MVP: model is the same regardless prompt is top or bottom - view is different
+#define HH_CONFIG_CASE          "casesensitive"
+#define HH_CONFIG_REGEXP        "regexp"
+#define HH_CONFIG_SUBSTRING     "substring"
+#define HH_CONFIG_KEYWORDS      "keywords"
+#define HH_CONFIG_SORTING       "rawhistory"
+#define HH_CONFIG_FAVORITES     "favorites"
+#define HH_CONFIG_NOCONFIRM     "noconfirm"
+#define HH_CONFIG_VERBOSE_KILL  "verbose-kill"
 #define HH_CONFIG_PROMPT_BOTTOM "prompt-bottom"
-#define HH_CONFIG_BLACKLIST  "blacklist"
-#define HH_CONFIG_KEEP_PAGE  "keep-page"
-#define HH_CONFIG_DEBUG      "debug"
-#define HH_CONFIG_WARN       "warning"
+#define HH_CONFIG_BLACKLIST     "blacklist"
+#define HH_CONFIG_KEEP_PAGE     "keep-page"
+#define HH_CONFIG_DEBUG         "debug"
+#define HH_CONFIG_WARN          "warning"
 #define HH_CONFIG_BIG_KEYS_SKIP  "big-keys-skip"
 #define HH_CONFIG_BIG_KEYS_FLOOR "big-keys-floor"
 #define HH_CONFIG_BIG_KEYS_EXIT  "big-keys-exit"
@@ -132,8 +132,6 @@
 #define HH_CASE_INSENSITIVE  0
 #define HH_CASE_SENSITIVE    1
 
-#define SPACE_PADDING "                                                              "
-
 #ifdef DEBUG_KEYS
 #define LOGKEYS(Y,KEY) mvprintw(Y, 0, "Key: '%3d' / Char: '%c'", KEY, KEY); clrtoeol()
 #else
@@ -182,18 +180,28 @@ static const char *INSTALL_BASH_STRING=
         "\nexport HISTCONTROL=ignorespace   # leading space hides commands from history"
         "\nexport HISTFILESIZE=10000        # increase history file size (default is 500)"
         "\nexport HISTSIZE=${HISTFILESIZE}  # increase history size (default is 500)"
+        // PROMPT_COMMAND considerations:
+        //   history -a ... append NEW entries from memory to .bash_history (i.e. flush to file where HSTR reads commands)
+        //   history -n ... append NEW entries from .bash_history to memory i.e. NOT entire history reload
+        //   history -c ... CLEAR in memory history (keeps .bash_history content)
+        //   history -r ... append ALL entries from .bash_history to memory (useful to sync DIFFERENT Bash sessions)
+        // Conclusion:
+        //   -a -n ... Fastest and almost-consistent option i.e. there is efficiency/integrity trade-off.
+        //             It works correctly if memory entries are not deleted by HSTR. It doesn't synchronize history
+        //             across different Bash sessions.
+        //   -c -r ... Forces entire .bash_history to be reloaded (handles history deletes, synchronizes different Bash sessions)
         "\nexport PROMPT_COMMAND=\"history -a; history -n; ${PROMPT_COMMAND}\"   # mem/file sync"
         "\n# if this is interactive shell, then bind hh to Ctrl-r (for Vi mode check doc)"
 // IMPROVE hh (win10) vs. hstr (cygwin) binary on various platforms must be resolved
 #if defined(__MS_WSL__)
         // IMPROVE commands are NOT executed on return under win10 > consider hstr_utils changes
-        "\nfunction hstr_winwls {"
+        "\nfunction hstr_winwsl {"
         "\n  offset=${READLINE_POINT}"
         "\n  READLINE_POINT=0"
         "\n  { READLINE_LINE=$(</dev/tty hh ${READLINE_LINE:0:offset} 2>&1 1>&$hstrout); } {hstrout}>&1"
         "\n  READLINE_POINT=${#READLINE_LINE}"
         "\n}"
-        "\nif [[ $- =~ .*i.* ]]; then bind -x '\"\\C-r\": \"hstr_winwls\"'; fi"
+        "\nif [[ $- =~ .*i.* ]]; then bind -x '\"\\C-r\": \"hstr_winwsl\"'; fi"
 #elif defined(__CYGWIN__)
         "\nfunction hstr_cygwin {"
         "\n  offset=${READLINE_POINT}"
@@ -204,6 +212,8 @@ static const char *INSTALL_BASH_STRING=
         "\nif [[ $- =~ .*i.* ]]; then bind -x '\"\\C-r\": \"hstr_cygwin\"'; fi"
 #else
         "\nif [[ $- =~ .*i.* ]]; then bind '\"\\C-r\": \"\\C-a hh -- \\C-j\"'; fi"
+        "\n# if this is interactive shell, then bind 'kill last command' to Ctrl-x k"
+        "\nif [[ $- =~ .*i.* ]]; then bind '\"\\C-xk\": \"\\C-a hh -k \\C-j\"'; fi"
 #endif
         "\n\n";
 
@@ -212,7 +222,7 @@ static const char *INSTALL_ZSH_STRING=
         "\nexport HISTFILE=~/.zsh_history  # ensure history file visibility"
         "\nexport HH_CONFIG=hicolor        # get more colors"
         "\nbindkey -s \"\\C-r\" \"\\eqhh\\n\"     # bind hh to Ctrl-r (for Vi mode check doc)"
-        // TODO try variant with arg/pars separation
+        // TODO try variant with args/pars separation
         //"\nbindkey -s \"\\C-r\" \"\\eqhh --\\n\"     # bind hh to Ctrl-r (for Vi mode check doc)"
         // alternate binding options in zsh:
         //   bindkey -s '^R' '^Ahh ^M'
@@ -224,6 +234,7 @@ static const char *HELP_STRING=
         "\nShell history suggest box:"
         "\n"
         "\n  --favorites              -f ... show favorites view"
+        "\n  --kill-last-command      -k ... delete last command in history"
         "\n  --non-interactive        -n ... print filtered history and exit"
         "\n  --show-configuration     -s ... show configuration to be added to ~/.bashrc"
         "\n  --show-zsh-configuration -z ... show Zsh configuration to be added to ~/.zshrc"
@@ -236,7 +247,7 @@ static const char *HELP_STRING=
         "\n";
 
 static const char *VERSION_STRING=
-        "hh version \"1.24\" (2018-01-03T12:19:00)"
+        "hh version \"1.25\" (2018-05-06T19:30:00)"
         "\n";
 
 // TODO help screen - curses window (tig)
@@ -249,6 +260,7 @@ static const char *LABEL_HELP=
 
 static const struct option long_options[] = {
         {"favorites",              GETOPT_NO_ARGUMENT, NULL, 'f'},
+        {"kill-last-command",      GETOPT_NO_ARGUMENT, NULL, 'k'},
         {"version",                GETOPT_NO_ARGUMENT, NULL, 'V'},
         {"help",                   GETOPT_NO_ARGUMENT, NULL, 'h'},
         {"non-interactive",        GETOPT_NO_ARGUMENT, NULL, 'n'},
@@ -276,6 +288,7 @@ typedef struct {
     unsigned char theme;
     bool keepPage; // do NOT clear page w/ selection on HH exit
     bool noConfirm; // do NOT ask for confirmation on history entry delete
+    bool verboseKill; // write a message on delete of the last command in history
     int bigKeys;
     int debugLevel;
 
@@ -302,7 +315,7 @@ void hstr_init()
     hstr->selectionRegexpMatch=NULL;
     hstr->selectionSize=0;
 
-    hstr->historyMatch=HH_MATCH_SUBSTRING;
+    hstr->historyMatch=HH_MATCH_KEYWORDS;
     hstr->historyView=HH_VIEW_RANKING;
     hstr->caseSensitive=HH_CASE_INSENSITIVE;
 
@@ -379,6 +392,9 @@ void hstr_get_env_configuration(Hstr *hstr)
                 hstr->bigKeys=RADIX_BIG_KEYS_SKIP;
             }
         }
+        if(strstr(hstr_config,HH_CONFIG_VERBOSE_KILL)) {
+            hstr->verboseKill=true;
+        }
         if(strstr(hstr_config,HH_CONFIG_BLACKLIST)) {
             hstr->blacklist.useFile=true;
         }
@@ -458,10 +474,15 @@ void add_to_selection(Hstr *hstr, char *line, unsigned int *index)
 
 void print_help_label()
 {
+    int cursorX=getcurx(stdscr);
+    int cursorY=getcury(stdscr);
+
     char screenLine[CMDLINE_LNG];
     snprintf(screenLine, getmaxx(stdscr), "%s", LABEL_HELP);
     mvprintw(hstr->promptYHelp, 0, "%s", screenLine); clrtoeol();
     refresh();
+
+    move(cursorY, cursorX);
 }
 
 void print_confirm_delete(const char *cmd, Hstr *hstr)
@@ -476,7 +497,7 @@ void print_confirm_delete(const char *cmd, Hstr *hstr)
     mvprintw(hstr->promptYHelp, 0, "%s", screenLine);
     if(hstr->theme & HH_THEME_COLOR) {
         color_attr_off(A_BOLD);
-        color_attr_on(COLOR_PAIR(1));
+        color_attr_on(COLOR_PAIR(HH_COLOR_NORMAL));
     }
     clrtoeol();
     refresh();
@@ -494,7 +515,7 @@ void print_cmd_deleted_label(const char *cmd, int occurences, Hstr *hstr)
     mvprintw(hstr->promptYHelp, 0, "%s", screenLine);
     if(hstr->theme & HH_THEME_COLOR) {
         color_attr_off(A_BOLD);
-        color_attr_on(COLOR_PAIR(1));
+        color_attr_on(COLOR_PAIR(HH_COLOR_NORMAL));
     }
     clrtoeol();
     refresh();
@@ -511,7 +532,7 @@ void print_regexp_error(const char *errorMessage)
     mvprintw(hstr->promptYHelp, 0, "%s", screenLine);
     if(hstr->theme & HH_THEME_COLOR) {
         color_attr_off(A_BOLD);
-        color_attr_on(COLOR_PAIR(1));
+        color_attr_on(COLOR_PAIR(HH_COLOR_NORMAL));
     }
     clrtoeol();
     refresh();
@@ -528,7 +549,7 @@ void print_cmd_added_favorite_label(const char *cmd, Hstr *hstr)
     mvprintw(hstr->promptYHelp, 0, screenLine);
     if(hstr->theme & HH_THEME_COLOR) {
         color_attr_off(A_BOLD);
-        color_attr_on(COLOR_PAIR(1));
+        color_attr_on(COLOR_PAIR(HH_COLOR_NORMAL));
     }
     clrtoeol();
     refresh();
@@ -625,9 +646,9 @@ unsigned hstr_make_selection(char *prefix, HistoryItems *history, int maxSelecti
     bool regexpCompilationError=false;
     bool keywordsAllMatch;
     char *keywordsSavePtr=NULL;
-    char *keywordsToken;
-    char *keywordsParsedLine;
-    char *keywordsPointerToDelete;
+    char *keywordsToken=NULL;
+    char *keywordsParsedLine=NULL;
+    char *keywordsPointerToDelete=NULL;
     for(i=0; i<count && selectionCount<maxSelectionCount; i++) {
         if(source[i]) {
             if(!prefix || !strlen(prefix)) {
@@ -741,12 +762,12 @@ void print_selection_row(char *text, int y, int width, char *pattern)
         if(hstr->theme & HH_THEME_COLOR) {
             color_attr_on(COLOR_PAIR(HH_COLOR_MATCH));
         }
-        char* p;
-        char* pp;
-        char* keywordsSavePtr;
-        char* keywordsToken;
-        char* keywordsParsedLine;
-        char* keywordsPointerToDelete;
+        char* p=NULL;
+        char* pp=NULL;
+        char* keywordsSavePtr=NULL;
+        char* keywordsToken=NULL;
+        char* keywordsParsedLine=NULL;
+        char* keywordsPointerToDelete=NULL;
 
         switch(hstr->historyMatch) {
         case HH_MATCH_SUBSTRING:
@@ -804,17 +825,17 @@ void hstr_print_highlighted_selection_row(char *text, int y, int width, Hstr *hs
 {
     color_attr_on(A_BOLD);
     if(hstr->theme & HH_THEME_COLOR) {
-        color_attr_on(COLOR_PAIR(2));
+        color_attr_on(COLOR_PAIR(HH_COLOR_HIROW));
     } else {
         color_attr_on(A_REVERSE);
     }
     char screenLine[CMDLINE_LNG];
-    snprintf(screenLine, getmaxx(stdscr),
-            "%s%s" SPACE_PADDING SPACE_PADDING SPACE_PADDING,
-            (terminal_has_colors()?" ":">"), text);
+    snprintf(screenLine, getmaxx(stdscr)+1, "%s%-*.*s ",
+            (terminal_has_colors()?" ":">"),
+            getmaxx(stdscr)-2, getmaxx(stdscr)-2, text);
     mvprintw(y, 0, "%s", screenLine);
     if(hstr->theme & HH_THEME_COLOR) {
-        color_attr_on(COLOR_PAIR(1));
+        color_attr_on(COLOR_PAIR(HH_COLOR_NORMAL));
     } else {
         color_attr_off(A_REVERSE);
     }
@@ -1049,6 +1070,10 @@ void loop_to_select(Hstr *hstr)
             printDefaultLabel=FALSE;
         }
 
+        if(c == K_CTRL_R) {
+            c = (hstr->promptBottom ? K_CTRL_P : K_CTRL_N);
+        }
+
         switch (c) {
         case KEY_HOME:
             // avoids printing of wild chars in search prompt
@@ -1078,6 +1103,12 @@ void loop_to_select(Hstr *hstr)
                 printDefaultLabel=TRUE;
                 print_history_label();
 
+                if(hstr->selectionSize == 0) {
+                    // just update the cursor, there are no elements to select
+                    move(hstr->promptY, basex+strlen(pattern));
+                    break;
+                }
+
                 if(hstr->promptBottom) {
                     if(selectionCursorPosition <= hstr->promptYItemsEnd-hstr->selectionSize+1) {
                         selectionCursorPosition=hstr->promptYItemsEnd-hstr->selectionSize+1;
@@ -1152,6 +1183,7 @@ void loop_to_select(Hstr *hstr)
             break;
         case KEY_RESIZE:
             print_history_label();
+            maxHistoryItems=recalculate_max_history_items();
             result=hstr_print_selection(maxHistoryItems, pattern, hstr);
             print_history_label();
             selectionCursorPosition=SELECTION_CURSOR_IN_PROMPT;
@@ -1221,7 +1253,6 @@ void loop_to_select(Hstr *hstr)
             highlight_selection(selectionCursorPosition, previousSelectionCursorPosition, pattern, hstr);
             move(hstr->promptY, basex+strlen(pattern));
             break;
-        case K_CTRL_R:
         case KEY_DOWN:
         case K_CTRL_J:
         case K_CTRL_N:
@@ -1407,7 +1438,7 @@ void hstr_main(Hstr *hstr)
 void hstr_getopt(int argc, char **argv, Hstr *hstr)
 {
     int option_index = 0;
-    int option = getopt_long(argc, argv, "fVhnszb", long_options, &option_index);
+    int option = getopt_long(argc, argv, "fkVhnszb", long_options, &option_index);
     if(option != -1) {
         switch(option) {
         case 'f':
@@ -1416,6 +1447,12 @@ void hstr_getopt(int argc, char **argv, Hstr *hstr)
         case 'n':
             hstr->interactive=false;
             break;
+        case 'k':
+            if(history_mgmt_remove_last_history_entry(hstr->verboseKill)) {
+                exit(EXIT_SUCCESS);
+            } else {
+                exit(EXIT_FAILURE);
+            }
         case 'b':
             blacklist_load(&hstr->blacklist);
             blacklist_dump(&hstr->blacklist);
