@@ -16,6 +16,7 @@
  limitations under the License.
 */
 
+#include <ctype.h>
 #include <stdio.h>
 #include <limits.h>
 #include <readline/history.h>
@@ -49,7 +50,7 @@ unsigned history_ranking_function(unsigned rank, int newOccurenceOrder, size_t l
     return metrics;
 }
 
-char *get_history_file_name()
+char *get_history_file_name(void)
 {
     char *historyFile=getenv(ENV_VAR_HISTFILE);
     if(!historyFile || strlen(historyFile)==0) {
@@ -74,7 +75,7 @@ void dump_prioritized_history(HistoryItems *historyItems)
     printf("\n"); fflush(stdout);
 }
 
-int get_item_offset()
+int get_item_offset(void)
 {
     if(isZshParentShell()) {
         // In zsh history file, the format of item is
@@ -89,6 +90,24 @@ int get_item_offset()
     }
 }
 
+bool is_hist_timestamp(const char *line)
+{
+    // HISTTIMEFORMAT defined > ^#1234567890$
+
+    if(line[0] != '#') {
+        return false;
+    }
+
+    int i;
+    for(i = 1; line[i] != '\0'; ++i) {
+        if(!isdigit(line[i])) {
+            return false;
+        }
+    }
+
+    return (i >= 11);
+}
+
 HistoryItems *get_prioritized_history(int optionBigKeys, HashSet *blacklist)
 {
     using_history();
@@ -112,11 +131,6 @@ HistoryItems *get_prioritized_history(int optionBigKeys, HashSet *blacklist)
         radixsort_init(&rs, (radixMaxKeyEstimate<100000?100000:radixMaxKeyEstimate));
         rs.optionBigKeys=optionBigKeys;
 
-        regex_t regexp;
-        // HISTTIMEFORMAT defined > ^#1234567890$
-        const char *histtimeformatTimestamp = "^#[[:digit:]]\\{10,\\}$";
-        regexp_compile(&regexp, histtimeformatTimestamp);
-
         RankedHistoryItem *r;
         RadixItem *radixItem;
         HIST_ENTRY **historyList=history_list();
@@ -124,7 +138,7 @@ HistoryItems *get_prioritized_history(int optionBigKeys, HashSet *blacklist)
         int rawOffset=historyState->length-1, rawTimestamps=0;
         char *line;
         for(i=0; i<historyState->length; i++, rawOffset--) {
-            if(!regexp_match(&regexp, historyList[i]->line)) {
+            if(is_hist_timestamp(historyList[i]->line)) {
                 rawHistory[rawOffset]=0;
                 rawTimestamps++;
                 continue;
@@ -171,8 +185,6 @@ HistoryItems *get_prioritized_history(int optionBigKeys, HashSet *blacklist)
             }
         }
 
-        regfree(&regexp);
-
         DEBUG_RADIXSORT();
 
         RadixItem **prioritizedRadix=radixsort_dump(&rs);
@@ -202,18 +214,18 @@ HistoryItems *get_prioritized_history(int optionBigKeys, HashSet *blacklist)
     }
 }
 
-void free_prioritized_history()
+void free_prioritized_history(void)
 {
     free(prioritizedHistory->items);
     free(prioritizedHistory);
 }
 
-void history_mgmt_open()
+void history_mgmt_open(void)
 {
     dirty=false;
 }
 
-void history_clear_dirty()
+void history_clear_dirty(void)
 {
     dirty=false;
 }
@@ -224,13 +236,7 @@ int history_mgmt_remove_from_system_history(char *cmd)
     char *l;
     HISTORY_STATE *historyState=history_get_history_state();
 
-    // TODO refactor this code to have this in source exactly once
-    regex_t tsRegexp;
     regex_t zshRegexp;
-    // HISTTIMEFORMAT defined > ^#1234567890$
-    const char *histtimeformatTimestamp = "^#[[:digit:]]\\{10,\\}$";
-    regexp_compile(&tsRegexp, histtimeformatTimestamp);
-
     const char *zshTimestamp = "^: [[:digit:]]\\{10,\\}:[[:digit:]];.*";
     regexp_compile(&zshRegexp, zshTimestamp);
 
@@ -246,7 +252,7 @@ int history_mgmt_remove_from_system_history(char *cmd)
                if(offset>0) {
                    l=historyState->entries[offset-1]->line;
                    if(l && strlen(l)) {
-                       if(!regexp_match(&tsRegexp, l)) {
+                       if(is_hist_timestamp(l)) {
                            // TODO check that this delete doesn't cause mismatch of searched cmd to be deleted
                            free_history_entry(remove_history(offset-1));
                        }
@@ -263,6 +269,33 @@ int history_mgmt_remove_from_system_history(char *cmd)
     return occurences;
 }
 
+bool history_mgmt_remove_last_history_entry(bool verbose)
+{
+    using_history();
+
+    char *historyFile = get_history_file_name();
+    if(read_history(historyFile)!=0) {
+        fprintf(stderr, "\nUnable to read history file from '%s'!\n",historyFile);
+        exit(EXIT_FAILURE);
+    }
+    HISTORY_STATE *historyState=history_get_history_state();
+    // delete the last command + the command that was used to run HSTR
+    if(historyState->length > 1) {
+        // length is NOT updated on history entry removal
+        if(verbose) {
+            fprintf(stdout, "Deleting command '%s' from history\n", historyState->entries[historyState->length-2]->line);
+        }
+        free_history_entry(remove_history(historyState->length-1));
+        free_history_entry(remove_history(historyState->length-2));
+        write_history(get_history_file_name());
+        return true;
+    }
+    if(verbose) {
+        fprintf(stderr, "Unable to delete the last command from history.\n");
+    }
+    return false;
+}
+
 int history_mgmt_remove_from_raw(char *cmd, HistoryItems *history) {
     int occurences=history->rawCount;
     if(history->rawCount) {
@@ -291,7 +324,7 @@ int history_mgmt_remove_from_ranked(char *cmd, HistoryItems *history) {
     return occurences-history->count;
 }
 
-void history_mgmt_flush()
+void history_mgmt_flush(void)
 {
     if(dirty) {
         fill_terminal_input("history -r\n", false);
