@@ -21,6 +21,8 @@
 
 #ifdef __APPLE__
 #include <curses.h>
+#elif defined(__FreeBSD__)
+#include <ncurses.h>
 #else
 #include <ncursesw/curses.h>
 #endif
